import argparse
import os
import configparser
import sys
import urllib.request

from . import gitbook
from . import filesystem
from . import common
from . import markdown
from . import github

INTEGRATION_FILE = ".ygitbookintegration"
INTEGRATION_MODULE = "integration"
SUBMODULE_MODULE = "submodule"
COMMIT_UPDATE_SUBMODULES = "✨ Alt sayfalar güncellendi"

# def execute_integrate(path: str, option: str):
#     COMMANDS = f"""
#     echo "---
#     description: Sitede neler olup bittiğinin raporudur.
#     ---" > {os.path.join(path, "CHANGELOG.md")}
#     ygitchangelog -d >> {os.path.join(path, "CHANGELOG.md")}
#     bash github {path}
#     """
#
#     # subprocess.run(f"bash -c '{COMMANDS}'")


def convert_url_form(filestr: str, url: str):
    return filestr.replace("./", url + "/").replace("README.md", "").replace(".md", "")


def read_config(startpath: str, filepath: str) -> dict:
    try:
        config = configparser.ConfigParser(inline_comment_prefixes="#")
        config.read(os.path.join(startpath, filepath), encoding="utf-8")
    except:
        return None

    return config


def updateSubSummaries(config: dict, startpath: str, index: str, push=False) -> None:
    """Alt modülleri günceller

    Arguments:
        config {dict} -- Yapılandırma dosyası verileri
        startpath {str} -- Yapılandırma dosyasının dizini

    Keyword Arguments:
        index {str} -- Limit (default: {"Index"})
        push {bool} -- Otomatik olarak git push işlemi yapma (default: {False})
    """

    def add_description(content: str, description: str) -> str:
        if description:
            return gitbook.make_description(description) + content
        else:
            return content

    def fixTitle(content: str) -> str:
        title = f"# {markdown.read_first_link(content)[1]}"
        title_changed = False

        lines = content.split("\n")

        for i in range(len(lines)):
            count = lines[i].count("#")
            if count == 1:
                lines[i] = title
                title_changed = True
                break
            elif count >= 1:
                lines[0] = title
                title_changed = True
                break

        if not title_changed:
            lines[0] = title
            title_changed = True

        return "\n".join(lines)

    def fixUrls(content, root):
        fixedlines = []
        lines = content.split("\n")
        for line in lines:
            link = markdown.find_link(line)
            if link:
                path: str = link[2]
                if "http" not in path:
                    fixedpath = path.replace(".md", "").replace("README", "")
                    fixedpath = root + "/" + fixedpath
                    line = line.replace(path, fixedpath)

            fixedlines.append(line)

        return "\n".join(fixedlines)

    paths = []
    for name in config.sections():
        if name.split()[0] == SUBMODULE_MODULE:
            section = config[name]

            path = os.path.join(startpath, section['path'])
            url = section['url']
            root = section['root']

            description = None
            if "description" in section:
                description = section['description']

            until = None
            if "until" in section:
                until = section['until']

            content = gitbook.read_summary_from_url(url)

            substrings = markdown.generate_substrings(content, index)
            if substrings:
                content = substrings[0]

            content = fixTitle(content)
            content = fixUrls(content, root)
            content = add_description(content, description)

            if until:
                until = "## " + until
                content = content[:content.find(until)]

            filesystem.write_file(path, content)

            paths.append(path)

    if push:
        github.push_to_github(startpath, paths, COMMIT_UPDATE_SUBMODULES)


def parse_args():
    global parser
    if 'parser' in globals():
        return parser.parse_args()
    else:
        parser = argparse.ArgumentParser(
            description='Create `README.md` and `SUMMARY.md` file for GitBook synchronization',
        )
        parser.add_argument(
            'paths',
            nargs="+",
            metavar='paths',
            help='Projelerin yolları',
        )
        parser.add_argument(
            "--update",
            "-u",
            action="store_true",
            dest="update",
            help="Alt modüllerin summary'lerini güncelleme"
        )
        parser.add_argument(
            "--recreate",
            "-r",
            action="store_true",
            dest="recreate",
            help="Summary dosyasını baştan oluşturur"
        )
        parser.add_argument(
            "--generate",
            "-g",
            action="store_true",
            dest="generate",
            help="Markdown dışı dosyalar için README'ye bağlantılar oluşturma"
        )
        parser.add_argument(
            '--level-limit',
            '-ll',
            dest="level_limit",
            default=-1,
            help='Fonksiyonların çalışacağı en yüksek derinlik (0 => ana dizinde çalışır sadece)',
            type=int,
        )
        parser.add_argument(
            "--debug",
            "-d",
            action="store_true",
            dest="debug",
            help="Bilgilendirici metinleri gösterme"
        )
        parser.add_argument(
            "--store",
            "-s",
            action="store_true",
            dest="store",
            help="Son komutu hızlı kullanım için yapılandırma dosyasında saklar"
        )
        parser.add_argument(
            "--push",
            "-p",
            action="store_true",
            dest="push",
            help="İşlem sonrası GitHub'a otomatik pushlar"
        )
        # WARN: Kullanışsız parametreler
        parser.add_argument(
            '--index',
            '-ix',
            dest="indexStr",
            default="YPackage.YGitbookIntegration-tarafından-otomatik-oluşturulmuştur",
            help='Generated string will be inserted between given indexes',
            type=str,
        )
        # BUG: Bu yapı çalışmaz, nargs olması lazım
        parser.add_argument(
            '--privates',
            "-pp",
            dest="privates",
            default=["res", "__pycache__"],
            help='List of folder names that you dont want to add',
            type=list
        )
        parser.add_argument(
            "--footer-path",
            '-fp',
            dest="footerPath",
            help="Append the given file to end of the output. "
        )
        parser.add_argument(
            '--new-index',
            '-nix',
            dest="newIndex",
            default=None,
            help='Generated string will be inserted between given indexes with changing the indexes',
            type=str,
        )

        return parser.parse_args()


def integrate(paths, safe=False):
    new_args_start_index = (len(paths) + 1)
    override = len(sys.argv) > new_args_start_index

    for path in paths:
        if not safe and path[-2:] == "/*":
            dirname = path[:-2]

            new_paths = []
            for path in os.listdir(dirname):
                new_path = os.path.join(dirname, path)
                if os.path.isdir(new_path):
                    new_paths.append(new_path)

            integrate(new_paths, safe=True)

        elif os.path.isdir(path):
            config = read_config(path, INTEGRATION_FILE)

            if not override:
                new_args = ""
                for section in config.sections():
                    if section.split()[0] == INTEGRATION_MODULE:
                        new_args = config[section]["args"].replace("\"", "")
                        break

                if new_args:
                    sys.argv = [__file__, path] + new_args.split()
                    new_args_start_index = 2  # len(paths) + 1
                else:
                    print(f"{os.path.basename(path)} için entegrasyon özelliği mevcut değil")
                    print(
                        f"    `{INTEGRATION_FILE}` dosyası içerisindeki `integration` alanında `args` özelliği yok")
                    continue

            args = parse_args()
            PRIVATES, INDEX_STR, NEW_INDEX_STR, FOOTER_PATH, LEVEL_LIMIT, UPDATE, RECREATE, GENERATE, STORE, PUSH = args.privates, args.indexStr, args.newIndex, args.footerPath, args.level_limit, args.update, args.recreate, args.generate, args.store, args.push

            if STORE:
                last_args = sys.argv[new_args_start_index:]
                last_args = " ".join(last_args)

                ifpath = os.path.join(path, INTEGRATION_FILE)
                if not os.path.exists(ifpath):
                    with open(ifpath, "w", encoding="utf-8") as file:
                        file.write(f'[{INTEGRATION_MODULE} "auto"]\n')
                        file.write(f'\targs = {last_args}')
                else:
                    for name in config.sections():
                        if name.split()[0] == INTEGRATION_MODULE:
                            section = config[name]
                            section["args"] = last_args

                    with open(ifpath, "w", encoding="utf-8") as configfile:
                        config.write(configfile)

            if GENERATE:
                gitbook.generate_readmes(
                    path, privates=PRIVATES, index=INDEX_STR,
                    new_index=NEW_INDEX_STR, level_limit=LEVEL_LIMIT
                )

            if RECREATE:
                filestr = gitbook.generate_summary_filestr(
                    path, level_limit=LEVEL_LIMIT, privates=PRIVATES,
                    footer_path=FOOTER_PATH
                )
                gitbook.create_summary_file(path)
                gitbook.insert_summary_file(
                    path, filestr, index=INDEX_STR, new_index=NEW_INDEX_STR
                )

            if UPDATE:
                updateSubSummaries(config, path, INDEX_STR, push=PUSH)

        else:
            print(f"Hatalı yol: {path}")


def generate_changelog():
    pass


def main():
    args = parse_args()
    integrate(args.paths)


if __name__ == "__main__":
    main()



import os
import re
from enum import Enum
from urllib.parse import quote

from .filesystem import (
    insert_file as fs_insert_file
)
from .common import (
    generate_substrings as c_generate_substrings
)

LINK_REGEX = r"\[([^\[]+)\]\((.*)\)"


class SpecialFile(Enum):
    README_FILE = "README.md"
    CHANGELOG_FILE = "CHANGELOG.md"
    CODE_OF_CONTACT = "CODE_OF_CONDUCT.md"
    CONTRIBUTING_FILE = "CONTRIBUTING.md"
    LICANSE_FILE = "Licanse.md"

    def get_filepath(self, root=os.getcwd(), symbolic=False) -> str:
        if symbolic:
            return os.path.join(root, self.value)

        for path in os.listdir(root):
            path = os.path.join(root, path)
            if os.path.isfile(path) and os.path.basename(path) == self.value:
                return os.path.join(root, self.value)

# DEV: Figure out index string in markdown file


def remove_extension(filepath: str) -> str:
    """Dosya uzantısını kaldırma

    Args:
        filepath (str): Dosya yolu

    Returns:
        str: Uzantsız dosya yolu
    """

    filepath, _ = os.path.splitext(filepath)
    return filepath


def barename(path: str) -> str:
    """Dosya veya dizin yolundan, yol ve uzantıyı temizleme

    Args:
        filepath (str): Dosya yolu

    Returns:
        str: Sadece dosya ismi
    """

    pathname = path
    if os.path.isfile(path):
        pathname = remove_extension(path)

    pathname = os.path.basename(pathname)

    return pathname


def encodedpath(path: str) -> str:
    """ Verilen yolu url formatında kodlama

    Windows için gelen '\\' karakteri '/' karakterine çevrilir

    Args:
        pathname (str): Yol ismi

    Returns:
        str: Kodlanmış metin
    """

    return quote(path.replace("\\", "/"))


def create_indent(level, size=2):
    return ' ' * size * (level)


def make_linkstr(header, link):
    return f"- [{header}]({link})\n"


def create_link(path: str, header: str = None, root: str = os.getcwd(), ilvl=0, isize=2) -> str:
    """Verilen yola uygun kodlanmış markdown linki oluşturma

    Arguments:
        path {str} -- Yol

    Keyword Arguments:
        header {str} -- Link başlığı (default: {None})
        root {str} -- Ana dizin (default: {os.getcwd()})
        ilvl {int} -- Girinti seviyesi (default: {0})
        isize {int} -- Girinti birim uzunluğu (default: {2})

    Returns:
        str -- Girintili markdown bağlatısı
    """

    pathname = barename(path) if not header else header
    path = os.path.relpath(path, root)
    path = encodedpath(path)

    indent = create_indent(ilvl, size=isize)
    linkstr = make_linkstr(pathname, path)

    return '{}{}'.format(indent, linkstr)


def create_header(name: str, headerlvl: int) -> str:
    return "#" * headerlvl + f" {name}\n\n"


def read_first_header(filepath):
    header = ""
    with open(filepath, "r", encoding="utf-8") as file:
        for line in file:
            if line.startswith("#"):
                header = line.strip().replace("# ", "")
                break

    if "<!--" in header:
        header = header[:header.index("<!--")].strip()

    return header


def read_first_link(string: str) -> dict:
    """İlk link verisini döndürür

    Arguments:
        string {str} -- Link aranacak metin dosyası

    Returns:
        dict -- Link verisi

    Examples:
        >>> result[0] = "[name](url)"
        >>> result[1] = "name"
        >>> result[2] = "url"
    """
    return re.search(f"{LINK_REGEX}|$", string)


def remove_title(string: str) -> str:
    lines = string.split("\n")
    for line in lines:
        if line.count("#") == 1:
            lines.remove(line)
            break
        # 1'den fazla gelirse title yok demektir
        elif line.count("#") >= 1:
            break

    return "\n".join(lines)


def find_link(line: str) -> dict:
    """Varsa link verisini döndürür

    Arguments:
        string {str} -- Bağlantı aranacak metin dosyası

    Returns:
        dict -- Bulunan bağlantılar

    Examples:
        >>> result[0] = "[name](url)"
        >>> result[1] = "name"
        >>> result[2] = "url"
    """
    return re.search(LINK_REGEX, line)


def findall_links(string: str) -> dict:
    """Tüm linkleri döndürür

    Arguments:
        string {str} -- Bağlantı aranacak metin dosyası

    Returns:
        dict -- Bulunan bağlantılar

    Examples:
        >>> result[0] = ('name', 'url')
        >>> result[0][1] = "name"
        >>> result[0][1] = "url"
    """
    return re.findall(LINK_REGEX, string)


def generate_filelink(fpath: str, startpath: str = os.getcwd(), header: str = None, ilvl=0, isize=2) -> str:
    """Dosya için markdown linki oluşturur

    Arguments:
        path {str} -- Yol

    Keyword Arguments:
        header {str} -- Link başlığı (default: {None})
        root {str} -- Ana dizin (default: {os.getcwd()})
        ilvl {int} -- Girinti seviyesi (default: {0})
        isize {int} -- Girinti birim uzunluğu (default: {2})

    Returns:
        str -- Girintili markdown bağlatısı
    """
    if not header:
        header = read_first_header(fpath)

    return create_link(fpath, header=header, root=startpath, isize=isize, ilvl=ilvl)


def generate_dirlink(root: str, startpath: str = os.getcwd(), ilvl=0, isize=2) -> str:
    """Dizin için markdown linki oluşturur
    README.md varsa ona bağlantı verir

    Arguments:
        path {str} -- Yol

    Keyword Arguments:
        header {str} -- Link başlığı (default: {None})
        root {str} -- Ana dizin (default: {os.getcwd()})
        ilvl {int} -- Girinti seviyesi (default: {0})
        isize {int} -- Girinti birim uzunluğu (default: {2})

    Returns:
        str -- Girintili markdown bağlatısı
    """

    dirlink = ""
    readme_path = SpecialFile.README_FILE.get_filepath(root)
    if readme_path:
        header = read_first_header(readme_path)
        dirlink = create_link(
            readme_path, header=header, root=startpath, isize=isize, ilvl=ilvl
        )
    else:
        dirlink = create_link(root, root=startpath, isize=isize, ilvl=ilvl)

    return dirlink


def make_comment(string):
    return f"<!--{string}-->"


def insert_file(filepath, string, index, force=False, fileheader=None, new_index=None):
    if force and not os.path.exists(filepath):
        create_markdown_file(filepath, header=fileheader)

    index = make_comment(index)

    if bool(new_index):
        new_index = make_comment(new_index)

    fs_insert_file(filepath, string, index, new_index)


def create_markdown_file(filepath, header=None):
    if not bool(header):
        header = os.path.basename(filepath)

    with open(filepath, "w", encoding="utf-8") as file:
        file.write(create_header(header,  1))


def generate_substrings(content, index):
    index = make_comment(index)
    return c_generate_substrings(content, index)



import os
from .filesystem import (
    find_level,
    listdir_grouped,
    readFileWithURL
)
from .markdown import (
    make_linkstr,
    create_header,
    insert_file,
    create_markdown_file,
    encodedpath,
    create_link,
    generate_filelink,
    generate_dirlink,
    SpecialFile
)

from .github import (
    get_github_raw_link,
    generate_raw_url_from_repo_url
)

DESCRIPTIPON_TEMPLATE = """---
description: {}
---

"""

SUMMARY_FILE = "SUMMARY.md"
SUMMARY_FILE_HEADER = "# Summary"
CHANGELOG_HEADER = u"👀 Neler değişti"
CONTRIBUTING_HEADER = u"💖 Katkıda Bulunma Rehberi"
GITHUB_USERNAME = "yedhrab"


def make_description(string: str) -> str:
    return DESCRIPTIPON_TEMPLATE.format(string)


def get_specialfile_header(specialfile: SpecialFile) -> str:
    if specialfile == SpecialFile.CHANGELOG_FILE:
        return CHANGELOG_HEADER
    elif specialfile == SpecialFile.CONTRIBUTING_FILE:
        return CONTRIBUTING_HEADER


def make_file_link(filepath: str, root: str = None, direct_link: bool = False) -> str:
    if root is None:
        root = os.path.dirname(
            filepath)

    if direct_link:
        return make_linkstr(os.path.basename(filepath), get_github_raw_link(GITHUB_USERNAME, filepath))
    else:
        return create_link(filepath, root=root)


def generate_fs_link(lines: list, root: str, startpath: str = None, level_limit: int = -1, privates=[]) -> list:
    # TODO: Buranın markdown'a aktarılması lazım
    # RES: Decalator kavramının araştırılması lazım olabilir

    def append_firstline(lines: list) -> list:
        headpath = SpecialFile.README_FILE.get_filepath(root=startpath)
        header_link = generate_filelink(headpath, startpath=startpath)
        lines.append(header_link)
        return lines

    def append_specialfiles(lines: list, *specialfiles: SpecialFile) -> list:
        for specialfile in specialfiles:
            specialfile_path = specialfile.get_filepath(startpath)
            if specialfile_path:
                specialfile_link = generate_filelink(
                    specialfile_path,
                    header=get_specialfile_header(specialfile),
                    startpath=startpath,
                    ilvl=find_level(root, startpath) + 1,
                    isize=2
                )
                lines.append(specialfile_link)

        return lines

    def append_rootlink(lines: list, root: str, level: int) -> str:
        dirlink = generate_dirlink(root, startpath=startpath, ilvl=level, isize=2)
        lines.append(dirlink)
        return lines

    def append_filelink(lines: list, fpath: str, level: int):
        filelink = generate_filelink(fpath, startpath=startpath, ilvl=level, isize=2)
        lines.append(filelink)
        return lines

    def append_sublinks(lines: list, root: str, level: int, dirs_only=False):
        dirs, files = listdir_grouped(root, privates=privates)
        for dpath in dirs:
            lines = generate_fs_link(lines, dpath, startpath=startpath, privates=privates)

        if not dirs_only:
            for fpath in files:
                if ".md" in fpath and not SpecialFile.README_FILE.value in fpath:
                    lines = append_filelink(lines, fpath, level)

        return lines

    def append_links(lines: list):
        nonlocal startpath
        if startpath is None:
            startpath = os.path.normpath(root)
            lines = append_firstline(lines)
            lines = append_specialfiles(
                lines,
                SpecialFile.CHANGELOG_FILE,
                SpecialFile.CONTRIBUTING_FILE
            )

            level = find_level(root, startpath)
            if level_limit == -1 or level <= level_limit:
                lines = append_sublinks(lines, root, level + 1, dirs_only=True)
        else:
            level = find_level(root, startpath)
            if level_limit == -1 or level <= level_limit:
                lines = append_rootlink(lines, root, level)
                lines = append_sublinks(lines, root, level + 1)
        return lines

    return append_links(lines)


def get_summary_path(workdir: str):
    return os.path.join(workdir, SUMMARY_FILE)


def create_summary_file(workdir: str):
    filepath = get_summary_path(workdir)
    if not os.path.exists(filepath):
        with open(filepath, "w", encoding="utf-8") as file:
            file.write(SUMMARY_FILE_HEADER + "\n\n")


def generate_summary_filestr(workdir, level_limit: int = -1, privates=[], footer_path=None):
    def append_markdown_links(filestr: str):
        lines = generate_fs_link([], workdir, level_limit=level_limit, privates=privates)
        filestr += "".join(lines)
        return filestr

    def append_footer(filestr: str):
        if footer_path and os.path.isfile(footer_path):
            with open(os.path.join(footer_path), "r", encoding="utf-8") as file:
                filestr = "\n" + \
                    file.read()
        return filestr

    filestr = ""
    filestr = append_markdown_links(filestr)
    filestr = append_footer(filestr)

    return filestr


def insert_summary_file(workdir: str, filestr: str, index: str = "Index", new_index: str = None):
    FILEPATH = get_summary_path(workdir)
    insert_file(FILEPATH, filestr, index=index, new_index=new_index)


def generate_summary(workdir, level_limit: int = -1, privates=[], footer_path=None, index: str = "Index", new_index: str = None):
    filestr = generate_summary_filestr(
        workdir, level_limit=level_limit, privates=privates, footer_path=footer_path)
    insert_summary_file(workdir, filestr, index=index, new_index=new_index)


def generate_readmes(startpath, level_limit: int = -1, privates=[], index="Index", header="📂 Harici Dosyalar", new_index=None, clearify=False, direct_link: bool = False):
    # DEV: Ders notlarını README'ye ekleme direkt olarak dizin dosya oluştur
    def clear_private_dirs() -> list:
        return [d for d in dirs if not d.startswith('.') and d not in privates]

    def generate_links_for_nonmarkdowns(files) -> str:
        # OLD: Eski yapıdır, kontrollü bir şekilde kaldırılmalı
        filestr = ""
        links = []
        for f in files:
            if not ".md" in f:
                links.append(make_file_link(os.path.join(".", f), direct_link=direct_link))

        if bool(links):
            filestr = create_header(header, 2)
            filestr += "".join(links)

        return filestr

    def generate_markdown_files_for_subitems(startpath, clearify=False) -> str:
        # DEV: Aynı dizindekiler aynı dosya altına yazılacak
        for root, _, files in os.walk(startpath):
            if root == startpath:
                continue

            filestr = ""
            filepath = SpecialFile.README_FILE.get_filepath(root=root, symbolic=True)

            level = find_level(root, startpath)
            link_root = os.path.dirname(root)
            for _ in range(level - 1):
                link_root = os.path.dirname(link_root)

            for f in files:
                subfilepath = os.path.join(root, f)
                if not ".md" in f:
                    filestr += make_file_link(subfilepath, root=root, direct_link=direct_link)
                elif f != SpecialFile.README_FILE.value:
                    # DEV: Markdown dosyaları README'nin altına eklensin
                    filestr += create_link(subfilepath, root=link_root)

            if bool(filestr):
                if not os.path.exists(filepath):
                    oldfile = os.path.join(startpath, os.path.basename(root) + ".md")
                    if clearify:
                        try:
                            os.rename(oldfile, filepath)
                            print(oldfile + " -> " + filepath)
                        except Exception as e:
                            create_markdown_file(filepath, os.path.basename(root))
                            print(f"`{oldfile}` aktarılamadı. {e}")

                insert_file(filepath, filestr, index=index, force=True,
                            fileheader=os.path.basename(root), new_index=new_index)

    filestr = ""
    for root, dirs, files in os.walk(startpath):
        # Remove private directories
        dirs[:] = clear_private_dirs()

        # Skip startpath
        if root == startpath:
            continue

        filestr = ""
        level = find_level(root, startpath)
        if level_limit == -1 or level < level_limit:
            filestr = generate_links_for_nonmarkdowns(files)
        elif level == level_limit:  # 1
            generate_markdown_files_for_subitems(root, clearify=clearify)
        else:
            if clearify and level > level_limit:
                readme_path = SpecialFile.README_FILE.get_filepath(root=root, symbolic=True)
                if os.path.exists(readme_path):
                    os.remove(readme_path)
            continue

        if bool(filestr):
            filepath = SpecialFile.README_FILE.get_filepath(root=root, symbolic=True)
            fileheader = os.path.basename(root)
            insert_file(filepath, filestr, index=index, force=True,
                        fileheader=fileheader, new_index=new_index)


def get_summary_url_from_repo_url(repo_url):
    return generate_raw_url_from_repo_url(repo_url) + "/" + SUMMARY_FILE


def read_summary_from_url(repo_url):
    raw_url = get_summary_url_from_repo_url(repo_url)
    return readFileWithURL(raw_url)
